
import os
import io

import streamlit as st
import seaborn as sns
import matplotlib

from defermi import DefectsAnalysis
from defermi.gui.inputs import main_inputs, filter_entries
from defermi.gui.chempots import chempots
from defermi.gui.dos import dos
from defermi.gui.thermodynamics import thermodynamics
# from defermi.gui.plotter import plotter
from defermi.gui.utils import init_state_variable, save_session


def set_defaults():
    sns.set_theme(context='talk',style='whitegrid')

    st.session_state['fontsize'] = 16
    st.session_state['label_size'] = 16
    st.session_state['npoints'] = 80
    st.session_state['pressure_range'] = (1e-35,1e30)
    st.session_state['figsize'] = (8, 8)
   # st.session_state['fig_width_in_pixels'] = 700
    st.session_state['alpha'] = 0.0

    if "color_sequence" not in st.session_state:
        st.session_state['color_sequence'] = matplotlib.color_sequences['tab10']
        st.session_state['color_sequence'] += matplotlib.color_sequences['tab20']
        st.session_state['color_sequence'] += matplotlib.color_sequences['Pastel1']

    if st.session_state.da:
        st.session_state.da.sort_entries()
        full_da = DefectsAnalysis.from_dataframe(
                                        st.session_state['saved_dataframe'],
                                        band_gap=st.session_state.da.band_gap,
                                        vbm=st.session_state.da.vbm)
        st.session_state['color_dict'] = {name:st.session_state['color_sequence'][idx] for idx,name in enumerate(full_da.names)}
    return


pages_dict = {
    'home': st.Page('home.py',title='Home',icon=':material/home:'),
    'data': st.Page('data.py',title='Data',icon=':material/table:'), 
    'formation_energies': st.Page('formation_energies.py',title='Formation Energies',icon=':material/line_axis:'),  
    'doping': st.Page('doping.py',title='Doping',icon=':material/stacked_line_chart:'),
    'brouwer': st.Page('brouwer.py',title='Brouwer',icon=':material/ssid_chart:'),
    'fermi_level': st.Page('fermi_level.py',title='Fermi Level',icon=':material/show_chart:'),
    'ctls': st.Page('ctls.py',title='Charge Transition Levels',icon=':material/insert_chart:'),
    'binding_energies': st.Page('binding_energies.py',title='Binding Energies',icon=':material/stacked_line_chart:')     
}


st.markdown("""
<style>
/* Set sidebar max-width */
[data-testid="stSidebar"] {
    width: 800px;
    min-width: 500;
    max-width: 900px;
}
</style>
""", unsafe_allow_html=True)


with st.sidebar:
    main_inputs()
    filter_entries()
    chempots()

    dos()
    thermodynamics()
    
set_defaults()

# exclude binding energies if there are no complexes
pages = []
for k,v in pages_dict.items():
    if k == 'binding_energies':
        if st.session_state.da:
            if 'DefectComplex' in st.session_state.da.types:
                pages.append(v)
    else:
        pages.append(v)

page = st.navigation(pages,expanded=True)
init_state_variable('session_name',value='session')
filename = st.session_state['session_name'] + '.defermi'
save_session(filename)
page.run()







import streamlit as st
import matplotlib.pyplot as plt

from defermi.gui.brouwer import get_pO2_vs_fermi_level_figure
from defermi.gui.ctls import CTLsPlotter
from defermi.gui.doping import get_doping_vs_fermi_level_figure
from defermi.gui.formation_energies import FormationEnergiesPlotter
from defermi.gui.utils import svg_logo, init_state_variable, insert_space


st.set_page_config(layout="wide")
cols = st.columns(3)
with cols[1]:
    st.image(svg_logo,width=300)

st.divider()
cols = st.columns(2)
with cols[0]:
    fig = None
    if 'formation_energies_figure' in st.session_state:
        fig = st.session_state['formation_energies_figure']
    elif st.session_state.da:
        plotter = FormationEnergiesPlotter(st.session_state.da) 
        entries,colors = plotter.get_entries_and_colors(filter_names=False)
        fig = plotter.get_figure(entries,colors)
    if fig:
        fig.gca().set_title('')
        subcols = st.columns([0.32,0.68])
        with subcols[1]:
            st.markdown('#### Formation Energies')
        st.pyplot(fig, clear_figure=False, width="content")

with cols[1]:
    fig = None
    if 'ctls_figure' in st.session_state:
        fig = st.session_state['ctls_figure']
    elif st.session_state.da:
        plotter = CTLsPlotter(st.session_state.da) 
        entries = plotter.get_entries(filter_names=False)
        fig = plotter.get_figure(entries)
    if fig:
        fig.gca().set_title('')
        subcols = st.columns([0.25,0.75])
        with subcols[1]:
            st.markdown('#### Charge Transition Levels')
        st.pyplot(fig, clear_figure=False, width="content")


with cols[0]:
    st.write('')
    if 'doping_diagram_figure' in st.session_state:
        fig = st.session_state['doping_diagram_figure']
        fig.gca().set_title('')
        subcols = st.columns([0.37,0.63])
        with subcols[1]:
            st.markdown('#### Doping Diagram')
        st.pyplot(fig, clear_figure=False, width="content")

    

with cols[1]:
    insert_space(42)
    if 'brouwer_diagram_figure' in st.session_state:
        fig = st.session_state['brouwer_diagram_figure']
        fig.gca().set_title('')
        subcols = st.columns([0.35,0.65])
        with subcols[1]:
            st.markdown('#### Brouwer Diagram')
        st.pyplot(fig, clear_figure=False, width="content")










import streamlit as st

from defermi.gui.info import dataset_info
from defermi.gui.utils import init_state_variable


def main():
    st.title("Data")
    st.set_page_config(layout="wide")

    if st.session_state.da:
        cols = st.columns([0.1,0.1,0.7,0.1])
        with cols[0]:
            init_state_variable('edit_dataframe',value=False)
            edit_dataframe = st.checkbox('Edit',key='widget_edit_dataframe',value=st.session_state['edit_dataframe'])
            st.session_state['edit_dataframe'] = edit_dataframe

        with cols[1]:
            def reset_dataframes():
                for k in ['dataframe', 'df_complete','saved_dataframe']:
                    if k in st.session_state:
                        del st.session_state[k]
                st.session_state['edit_dataframe'] = False
                st.session_state['widget_edit_dataframe'] = False
                return 
            st.button('Reset',key='widget_reset_da',on_click=reset_dataframes)

        with cols[2]:
            csv_str = st.session_state.da.to_dataframe(include_data=False,include_structures=False).to_csv(index=False)
            filename = st.session_state['session_name'] + '_dataset.csv'
            st.download_button(
                label="💾 Save csv",
                data=csv_str,
                file_name=filename,
                mime="test/csv")   
        with cols[3]:
            with st.popover(label='ℹ️',help='Info',type='tertiary'):
                pass
                st.write(dataset_info)

        if st.session_state['edit_dataframe']:
            edited_df = st.data_editor(
                            st.session_state['df_complete'], 
                            column_config={
                                'Include':st.column_config.CheckboxColumn()
                            },
                            hide_index=True,
                            num_rows='dynamic',
                            height='stretch',
                            key='widget_data_editor')
            
            #st.write(edited_df)
            edited_df = edited_df.dropna() # exclude rows with NaN
            st.session_state['saved_dataframe'] = edited_df
            df_to_import = edited_df[edited_df["Include"] == True] # keep only selected rows
            st.session_state['dataframe'] = df_to_import

            st.session_state.pop('formation_energies_figure',None)

        else:
            st.session_state['df_complete'] = st.session_state['saved_dataframe']
            st.dataframe(st.session_state['saved_dataframe'],hide_index=True,height='stretch')

    else:
        st.warning('Dataset is empty')
    
if __name__ == '__main__':
    main()



import io
import streamlit as st
import json
import pandas as pd

from monty.json import jsanitize, MontyEncoder, MontyDecoder


def init_state_variable(key,value=None):
    if key not in st.session_state:
        st.session_state[key] = value

def reset_session():
    st.session_state.clear()
    return

def insert_space(px=20):
    """Insert vertical space in pixels."""
    st.markdown(f"<div style='margin-top:{px}px;'></div>", unsafe_allow_html=True)


def widget_with_updating_state(function, key, widget_key=None, **kwargs):
    """
    Create widget with updating default values by using st.session_state

    Parameters
    ----------
    function : function
        Function to use as widget.
    key : str
        Key for st.session_state dictionary.
    widget_key : str
        Key to assign to widget. If None, 'widget_{key}' is used.
    kwargs : dict
        Kwargs to pass to widget function. 'on_change' and 'key' kwargs 
        are set by default.

    Returns
    -------
    var : 
        Output of widget function.
    """
    widget_key = widget_key or 'widget_' + key
    def update_var():
        st.session_state[key] = st.session_state[widget_key]
    
    if 'on_change' not in kwargs:
        kwargs['on_change'] = update_var
    kwargs['key'] = widget_key

    var = function(**kwargs)
    st.session_state[key] = var
    return var


def get_session_data():
    data = {k:v for k,v in st.session_state.items() if 'widget' not in k and 'figure' not in k}
    keys_to_delete = [
        'session_loaded',
        'session_name',
        'precursors',
        'external_defects',
        'edit_dataframe',
        'input_dataframe',
        'df_complete',
    ]
    for k in keys_to_delete:
        data.pop(k,None)
    return data


def save_session(filename):
    """Save Streamlit session state to a JSON file."""
    try:
        data = get_session_data()
        d = MontyEncoder().encode(data)

        # convert to pretty JSON string
        json_str = json.dumps(d, indent=2)

        # create a downloadable button
        st.download_button(
            label="💾 Save Session",
            data=json_str,
            file_name=filename,
            mime="application/json"
        )
        

    except Exception as e:
        st.error(f"Failed to prepare session download: {e}")


def load_session(uploaded_file):
    """Load Streamlit session state from JSON file."""
    data_bytes = uploaded_file.getvalue() # file content as bytes
    data_str = data_bytes.decode('utf-8') # decode bytes to string (JSON text)
    json_str = json.loads(data_str)
    
    d = MontyDecoder().decode(json_str)
    st.session_state.update(d)

    # Convert DataFrame back to original index after monty encode/decode
    data_df = st.session_state['saved_dataframe'].to_dict(orient='records')
    st.session_state['saved_dataframe'] = pd.DataFrame(data=data_df)



def reset_home_figures():
    keys = [
        'formation_energies_figure',
        'brouwer_diagram_figure',
        'doping_diagram_figure',
        'fermi_level_figure'
    ]
    for k in keys:
        st.session_state.pop(k,None)



def _filter_names(defect_names,key):

    names_key = f'names_{key}'
    init_state_variable(names_key,value=defect_names)
    init_state_variable(f'previous_names_{key}',value=defect_names)
    default = st.session_state[names_key]
    for name in st.session_state[names_key]:
        if name not in defect_names:
            default = defect_names
            break
    for name in defect_names:
        if name not in st.session_state[f'previous_names_{key}']:
            default.append(name)
    names = widget_with_updating_state(function=st.multiselect, key=names_key,label='Names',
                                    options=defect_names, default=default)
    st.session_state[f'previous_names_{key}'] = defect_names
    
    return names




def _filter_concentrations(defect_concentrations,key='brouwer'):

    output_key = f'output_{key}'
    init_state_variable(output_key,value='total')
    options = ['total','stable','all']
    index = options.index(st.session_state[output_key])
    output = widget_with_updating_state(function=st.radio,
                                        key=output_key,
                                        label='Concentrations style',
                                        options=options,
                                        index=index,
                                        horizontal=True)

    # select names
    conc_names = defect_concentrations.names
    names = _filter_names(defect_names=conc_names,key=key)

    # set consistent colors
    for idx,name in enumerate(names):
        if name not in st.session_state['color_dict'].keys():
            st.session_state['color_dict'][name] = st.session_state['color_sequence'][idx]
            for c in st.session_state['color_sequence']:
                if c not in st.session_state['color_dict'].values():
                    st.session_state['color_dict'][name] = c
                    break
    ordered_names = []
    for c in defect_concentrations.select_concentrations(names=names): # use plotting order
        if c.name not in ordered_names:
            ordered_names.append(c.name)
    colors = [st.session_state.color_dict[name] for name in ordered_names]

    # set charges and reset colors
    charges=None
    if output=='all':
        charges_key = f'charges_str_{key}'
        init_state_variable(charges_key,value=None)
        colors=None
        charges_str = st.text_input(label='Charges (q1,q2,...)',value=st.session_state[charges_key],key=f'widget_{charges_key}')
        st.session_state[charges_key] = charges_str
        if charges_str:
            charges = []
            for s in charges_str.split(','):
                charges.append(float(s))

    return output, names, charges, colors




def _get_axis_limits_with_widgets(label, key, default, boundaries):
    """
    Create widgets with axis limits that persist through session changes.
    Values are stored in `st.session_state`.

    Parameters
    ----------
    label : (str)
        Label to pass to widget.
    key : (str)
        String to pass to widget key.
    default : (tuple)
        Default value for axis limit.
    boundaries_ : tuple
        Max and min value for `st.slider` for axis.

    Returns
    -------
    set_lim : bool
        `st.checkbox` output for axis limit.
    lim : tuple
        `st.slider` output for axis limit.
    """
    lim_label = f'{label}_{key}'
    set_lim_label = 'set_'+ lim_label
    

    if set_lim_label not in st.session_state:
        st.session_state[set_lim_label] = False
    if lim_label not in st.session_state:
        st.session_state[lim_label] = default

    subcols = st.columns([0.3,0.7])
    with subcols[0]:
        set_lim = st.checkbox(label,value=st.session_state[set_lim_label],label_visibility='visible', key=f'widget_{set_lim_label}')
        st.session_state[set_lim_label] = set_lim
    with subcols[1]:
        disabled = not set_lim
        def update_default_lim(): 
            st.session_state[lim_label] = st.session_state[f'widget_{lim_label}']
        lim = st.slider(
                            label,
                            min_value=boundaries[0],
                            max_value=boundaries[1],
                            value=st.session_state[lim_label],
                            label_visibility='collapsed',
                            key=f'widget_{lim_label}',
                            disabled=disabled,
                            on_change=update_default_lim)  
        st.session_state[lim_label] = lim

    return set_lim, lim


def download_plot(fig,filename):
    # Convert the plot to PNG in memory
    buf = io.BytesIO()
    fig.savefig(buf, format="pdf",bbox_inches='tight')
    buf.seek(0)

    filename = st.session_state['session_name'] + '_' + filename
    # Add a download button
    st.download_button(
        label="💾 Save plot",
        data=buf,
        file_name=filename,
        mime="pdf"
    )


svg_logo = """
<svg
   width="137.35327mm"
   height="26.927582mm"
   viewBox="0 0 137.35327 26.927582"
   version="1.1"
   id="svg5"
   inkscape:version="1.2.2 (b0a8486541, 2022-12-01)"
   sodipodi:docname="defermi_logo.svg"
   inkscape:export-filename="defermi_logo.png"
   inkscape:export-xdpi="300"
   inkscape:export-ydpi="300"
   xmlns:inkscape="http://www.inkscape.org/namespaces/inkscape"
   xmlns:sodipodi="http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd"
   xmlns="http://www.w3.org/2000/svg"
   xmlns:svg="http://www.w3.org/2000/svg">
  <sodipodi:namedview
     id="namedview7"
     pagecolor="#ffffff"
     bordercolor="#666666"
     borderopacity="1.0"
     inkscape:showpageshadow="2"
     inkscape:pageopacity="0.0"
     inkscape:pagecheckerboard="0"
     inkscape:deskcolor="#d1d1d1"
     inkscape:document-units="mm"
     showgrid="false"
     inkscape:zoom="1.4793368"
     inkscape:cx="281.88306"
     inkscape:cy="61.176061"
     inkscape:window-width="3774"
     inkscape:window-height="1531"
     inkscape:window-x="0"
     inkscape:window-y="0"
     inkscape:window-maximized="1"
     inkscape:current-layer="layer1" />
  <defs
     id="defs2" />
  <g
     inkscape:label="Layer 1"
     inkscape:groupmode="layer"
     id="layer1"
     transform="translate(-35.33694,-51.548845)">
    <text
       xml:space="preserve"
       style="font-style:normal;font-variant:normal;font-weight:normal;font-stretch:normal;font-size:37.7137px;line-height:125%;font-family:'Latin Modern Sans Quotation';-inkscape-font-specification:'Latin Modern Sans Quotation, Normal';font-variant-ligatures:normal;font-variant-caps:normal;font-variant-numeric:normal;font-variant-east-asian:normal;text-align:start;letter-spacing:0px;word-spacing:0px;writing-mode:lr-tb;text-anchor:start;fill:#000000;fill-opacity:1;stroke:none;stroke-width:0.942845px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1"
       x="33.3004"
       y="78.099289"
       id="text113"><tspan
         sodipodi:role="line"
         id="tspan111"
         style="font-style:normal;font-variant:normal;font-weight:normal;font-stretch:normal;font-size:37.7137px;font-family:'Latin Modern Sans Quotation';-inkscape-font-specification:'Latin Modern Sans Quotation, Normal';font-variant-ligatures:normal;font-variant-caps:normal;font-variant-numeric:normal;font-variant-east-asian:normal;fill:#000080;stroke-width:0.942845px"
         x="33.3004"
         y="78.099289">d<tspan
   style="fill:#800000"
   id="tspan285">ef</tspan>ermi</tspan></text>
  </g>
</svg>
"""


import tempfile
import os
import time
import json
import io

import matplotlib
import streamlit as st
import pandas as pd



from defermi import DefectsAnalysis 
from defermi.gui.info import file_loader_info, band_gap_info
from defermi.gui.utils import load_session, init_state_variable, widget_with_updating_state


def reset_session():
    st.session_state.clear()
    return


def load_dataframe(uploaded_file):
    name = uploaded_file.name
    if name.endswith('.csv'):
        dataframe = pd.read_csv(uploaded_file) # Streamlit's UploadedFile can be read directly for csv
    elif name.endswith('.pkl'):
        dataframe = pd.read_pickle(io.BytesIO(uploaded_file.getvalue())) # pass the bytes for pickles
    else:
        st.error('Dataset format must be "csv" or "pkl"')
        return None
    return dataframe


def load_file(uploaded_file):
    init_state_variable('session_loaded',value=False)
    if uploaded_file:
        if ".defermi" in uploaded_file.name and not st.session_state['session_loaded']:
            load_session(uploaded_file) 
            st.session_state['session_loaded'] = True
            st.session_state['df_complete'] = st.session_state['saved_dataframe']
        elif '.defermi' not in uploaded_file.name and not st.session_state['session_loaded']:
            st.session_state['input_dataframe'] = load_dataframe(uploaded_file)


def band_gap_vbm_inputs():
    init_state_variable('band_gap',value=None)
    init_state_variable('vbm',value=0.0)

    cols = st.columns([0.45,0.45,0.1])
    with cols[0]:
        band_gap = st.number_input("Band gap (eV)", value=st.session_state['band_gap'], step=0.1, placeholder="Enter band gap", key='widget_band_gap')
        if band_gap is None:
            st.warning('Enter band gap to begin session')
        st.session_state['band_gap'] = band_gap
    with cols[1]:
        vbm = st.number_input("VBM (eV)", value=st.session_state['vbm'], step=0.1, key='widget_vbm')
        st.session_state['vbm'] = vbm
    with cols[2]:
        with st.popover(label='ℹ️',help='Info',type='tertiary'):
            st.write(band_gap_info)
    return 


def main_inputs():

    init_state_variable('da',value=None)

    st.markdown('## 📂 File')
    cols = st.columns([0.9,0.1])
    with cols[0]:
        uploaded_file = st.file_uploader("upload", type=["defermi","csv","pkl"], on_change=reset_session, label_visibility="collapsed")
        load_file(uploaded_file)
    with cols[1]:
        with st.popover(label='ℹ️',help='Info',type='tertiary'):
            st.write(file_loader_info)

    if uploaded_file:
        band_gap_vbm_inputs()
        if st.session_state['band_gap']:
            if not st.session_state['da']:
                df = st.session_state['input_dataframe']
                st.session_state['da'] = DefectsAnalysis.from_dataframe(
                                                                    df,
                                                                    band_gap=st.session_state['band_gap'],
                                                                    vbm=st.session_state['vbm'])
            else:
                st.session_state['da'].band_gap = st.session_state['band_gap']
                st.session_state['da'].vbm = st.session_state['vbm']

            if 'init' not in st.session_state:
                # message disappears after 1 second 
                msg = st.empty()
                msg.success("Dataset initialized")
                time.sleep(1)
                msg.empty()
                st.session_state.init = True
    
        st.divider()



def filter_entries():
    """
    GUI elements to filter defect entries in DefectsAnalysis
    """
    if st.session_state.da:
        st.session_state['da'].band_gap = st.session_state['band_gap']
        st.session_state['da'].vbm = st.session_state['vbm']
        init_state_variable('original_da',value=st.session_state.da.copy())
        
        df_complete = st.session_state.original_da.to_dataframe(include_data=False,include_structures=False) 
        df_complete['Include'] = [True for i in range(len(df_complete))]
        cols = ['Include'] + [col for col in df_complete.columns if col != 'Include']
        df_complete = df_complete[cols]

        init_state_variable('df_complete',value=df_complete)    
        init_state_variable('dataframe',value=df_complete)
        init_state_variable('saved_dataframe',value=df_complete)
        
        st.session_state.da = DefectsAnalysis.from_dataframe(
                                                    st.session_state['dataframe'],
                                                    band_gap=st.session_state['band_gap'],
                                                    vbm=st.session_state['vbm'],
                                                    include_data=False)  





import os

import defermi

def setup_gui(subparsers):

    subparsers_gui = subparsers.add_parser('gui',help='Launch GUI with streamlit')
    subparsers_gui.set_defaults(func=run_gui)


def run_gui(args):
    
    path_defermi = defermi.__file__
    path_gui_main = path_defermi.replace('__init__.py','gui/main.py')
    os.system(f'streamlit run {path_gui_main}')







import matplotlib.pyplot as plt
import streamlit as st
from contextlib import nullcontext

from defermi.plotter import plot_pO2_vs_fermi_level, plot_variable_species_vs_fermi_level
from defermi.gui.utils import download_plot


def main():

    st.set_page_config(layout="wide")
    st.title('Fermi Level')
    if st.session_state.da:
        da = st.session_state.da
        is_oxygen = 'O' in da.elements
        if is_oxygen:
            cols = st.columns(2)
            if 'brouwer_thermodata' in st.session_state:
                with cols[0]:
                    fig = st.session_state['fermi_level_brouwer_figure'] #_po2_vs_fermi_level_diagram(xlim,ylim)
                    st.pyplot(fig, clear_figure=False, width='content')
                    subcols = st.columns([0.4,0.6])
                    with subcols[1]:
                        download_plot(fig=fig,filename='fermi_level_brouwer.pdf')

        if 'doping_thermodata' in st.session_state:
            if st.session_state['doping_thermodata'] and st.session_state['dopant']:
                # no subcolumn if there is no brouwer diagram section 
                context = context = cols[1] if is_oxygen else nullcontext()
                with context:
                    fig = st.session_state['fermi_level_doping_figure']
                    st.pyplot(fig, clear_figure=False, width='content')
                    subcols = st.columns([0.4,0.6])
                    with subcols[1]:
                        download_plot(fig=fig,filename='fermi_level_doping.pdf')

    else:
        st.warning('Dataset is empty')


if __name__ == '__main__':
    main()



import os
import io
import json

import streamlit as st
import seaborn as sns
import matplotlib
from monty.json import jsanitize, MontyDecoder

import defermi
from defermi import DefectsAnalysis
from defermi.gui.inputs import main_inputs, filter_entries, band_gap_vbm_inputs, reset_session
from defermi.gui.chempots import chempots
from defermi.gui.dos import dos
from defermi.gui.thermodynamics import thermodynamics
# from defermi.gui.plotter import plotter
from defermi.gui.utils import init_state_variable, save_session


def load_session_from_example(file_path):
    """Load Streamlit session state from JSON file."""
    try:
        if os.path.exists(file_path):
            with open(file_path, "r") as f:
                d = json.load(f)
            d = MontyDecoder().decode(d)
            st.session_state.update(d)
        else:
            st.warning(f"File not found: {file_path}")
    except Exception as e:
        st.error(f"Failed to load session: {e}")


def set_defaults():
    sns.set_theme(context='talk',style='whitegrid')

    st.session_state['fontsize'] = 16
    st.session_state['label_size'] = 16
    st.session_state['npoints'] = 80
    st.session_state['pressure_range'] = (1e-35,1e30)
    st.session_state['figsize'] = (8, 8)
   # st.session_state['fig_width_in_pixels'] = 700
    st.session_state['alpha'] = 0.0

    if "color_sequence" not in st.session_state:
        st.session_state['color_sequence'] = matplotlib.color_sequences['tab10']
        st.session_state['color_sequence'] += matplotlib.color_sequences['tab20']
        st.session_state['color_sequence'] += matplotlib.color_sequences['Pastel1']

    if st.session_state.da:
        st.session_state.da.sort_entries()
        full_da = DefectsAnalysis.from_dataframe(
                                        st.session_state['saved_dataframe'],
                                        band_gap=st.session_state.da.band_gap,
                                        vbm=st.session_state.da.vbm)
        st.session_state['color_dict'] = {name:st.session_state['color_sequence'][idx] for idx,name in enumerate(full_da.names)}
    return


pages_dict = {
    'home': st.Page('home.py',title='Home',icon=':material/home:'),
    'data': st.Page('data.py',title='Data',icon=':material/table:'), 
    'formation_energies': st.Page('formation_energies.py',title='Formation Energies',icon=':material/line_axis:'),  
    'doping': st.Page('doping.py',title='Doping',icon=':material/stacked_line_chart:'),
    'brouwer': st.Page('brouwer.py',title='Brouwer',icon=':material/ssid_chart:'),
    'fermi_level': st.Page('fermi_level.py',title='Fermi Level',icon=':material/show_chart:'),
    'ctls': st.Page('ctls.py',title='Charge Transition Levels',icon=':material/insert_chart:'),
    'binding_energies': st.Page('binding_energies.py',title='Binding Energies',icon=':material/stacked_line_chart:')     
}


st.markdown("""
<style>
/* Set sidebar max-width */
[data-testid="stSidebar"] {
    width: 800px;
    min-width: 500;
    max-width: 900px;
}
</style>
""", unsafe_allow_html=True)


with st.sidebar:
   # main_inputs()
    init_state_variable('session_loaded',value=False)
    if not st.session_state['session_loaded']:
        session_file = os.path.join(defermi.gui.__path__[0],'app_example.defermi')
        load_session_from_example(session_file)
        st.session_state['session_loaded'] = True
    filter_entries()
    chempots()

    dos()
    thermodynamics()
    
set_defaults()

# exclude binding energies if there are no complexes
pages = []
for k,v in pages_dict.items():
    if k == 'binding_energies':
        if st.session_state.da:
            if 'DefectComplex' in st.session_state.da.types:
                pages.append(v)
    else:
        pages.append(v)

page = st.navigation(pages,expanded=True)
init_state_variable('session_name',value='session')
filename = st.session_state['session_name'] + '.defermi'
save_session(filename)
page.run()




